import io
import json
import os
from contextlib import contextmanager, redirect_stderr, redirect_stdout
from datetime import datetime
from pathlib import Path
from textwrap import dedent
from time import monotonic

import rich
import typer
from dotenv import load_dotenv
from rich.progress import Progress, SpinnerColumn, TextColumn

from aiai.code_analyzer import CodeAnalyzer
from aiai.optimizer.contextualizer import AgentContext, generate_context
from aiai.optimizer.rule_extractor import generate_rules_and_tips
from aiai.optimizer.rule_locator import RuleLocator
from aiai.runner.batch_runner import BatchRunner
from aiai.runner.py_script_tracer import PyScriptTracer
from aiai.synthesizer.data import generate_data
from aiai.synthesizer.evals import EvalGenerator, RulesEval, SyntheticEvalRunner
from aiai.utils import log_event, log_init, reset_db, setup_django

# Use absolute imports within the package

# ---------------------------------------------------------------------------
# Utility: silence stdout/stderr and loading spinner
# ---------------------------------------------------------------------------

cwd = Path.cwd()


@contextmanager
def silence():
    """Temporarily suppress *all* stdout / stderr output (prints, progress bars, etc.)."""
    stdout, stderr = io.StringIO(), io.StringIO()
    with redirect_stdout(stdout), redirect_stderr(stderr):
        yield stdout, stderr


@contextmanager
def loading(message: str, silent: bool = True):
    """Show pretty Rich spinner while silencing inner output."""
    start_at = monotonic()
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        transient=True,
    ) as progress:
        progress.add_task(description=message, total=None)
        if silent:
            with silence():
                yield
        else:
            yield
    duration = monotonic() - start_at
    typer.secho(f"✅ {message} completed in {duration:.2f}s", fg=typer.colors.GREEN)


def analyze_code(file: Path, model: str) -> AgentContext:
    from aiai.app.models import FunctionInfo

    CodeAnalyzer().analyze_from_file(file, save_to_db=True)
    source_code = json.dumps(
        {name: source for name, source in FunctionInfo.objects.values_list("name", "source_code")},
    )
    context = generate_context(source_code=source_code, model=model)
    rich.print_json(context.analysis.model_dump_json())
    return context


def generate_optimization_report(code_mods: list[dict], write_to: Path):
    """Generate a markdown report from optimization results."""
    md_lines = ["# Optimization Results\n"]

    # Build markdown content
    for code_mod in code_mods:
        file = code_mod["target"]["file_path"]
        md_lines.append(f"## File: {file}\n")
        md_lines.append("Source:")
        md_lines.append(f"```python\n{code_mod['target']['source_code']}\n```")
        md_lines.append("Optimizations:")
        if code_mod["mods"].get("always"):
            md_lines.append("<always>")
            for i, rule in enumerate(code_mod["mods"]["always"]):
                md_lines.append(f"  {i + 1}. {rule}")
            md_lines.append("</always>")
        if code_mod["mods"].get("never"):
            md_lines.append("<never>")
            for i, rule in enumerate(code_mod["mods"]["never"]):
                md_lines.append(f"  {i + 1}. {rule}")
            md_lines.append("</never>")
        if code_mod["mods"].get("tips"):
            md_lines.append("<tips>")
            for i, rule in enumerate(code_mod["mods"]["tips"]):
                md_lines.append(f"  {i + 1}. {rule}")
            md_lines.append("</tips>")
        md_lines.append("\n")

    content = "\n".join(md_lines)
    write_to.write_text(content)
    return content


def _validate_entrypoint(entrypoint: Path):
    if not entrypoint.exists():
        typer.secho(f"❌ Entrypoint file not found: {entrypoint}", fg=typer.colors.RED)
        raise typer.Exit(code=1)
    with loading("Validating entrypoint…"):
        with PyScriptTracer(entrypoint) as tracer:
            tracer()


def _optimization_run(
    entrypoint: Path,
    rules_eval: RulesEval,
    context: AgentContext,
    evaluator: str,
    optimizer: str,
    synthesizer: str,
    examples: int,
    concurrency: int,
    seed: int,
):
    """Run a single optimisation pass (no epochs)."""
    # Import models here, after setup_django has run
    from aiai.app.models import SyntheticDatum

    # ------------------------------------------------------------------
    # Generate synthetic data (or reuse if already present)
    # ------------------------------------------------------------------
    data = list(SyntheticDatum.objects.all())
    if not data:
        with loading(f"Generating {examples} synthetic inputs…"):
            data = generate_data(context, examples, seed, model=synthesizer)

    batch_runner = BatchRunner(
        script=entrypoint,
        data=[d.input_data for d in data],
        eval=SyntheticEvalRunner(rules_eval, model=evaluator),
        concurrency=concurrency,
    )
    with loading("Evaluating..."):
        eval_runs = batch_runner.perform()

    import pandas as pd

    df = pd.DataFrame([(e.trace_id, e.reward["reward"]) for e in eval_runs])
    stats = {stat: float(getattr(df[1], stat)()) for stat in ["min", "mean", "median", "max", "std"]}
    rich.print_json(data=stats)
    log_event("stats", **stats)

    log_event(
        "optimization",
        evaluator=evaluator,
        optimizer=optimizer,
        synthesizer=synthesizer,
        examples=examples,
    )
    with loading("Optimizing…"):
        rules_and_tips = generate_rules_and_tips(context=context, model=optimizer)

    # ------------------------------------------------------------------
    # Rule placement
    # ------------------------------------------------------------------
    with loading("Generating code modifications…"):
        code_mods = RuleLocator(rules_and_tips, model=optimizer).perform()

    # ------------------------------------------------------------------
    # Display placements & save to markdown
    # ------------------------------------------------------------------
    if not code_mods:
        typer.echo("⚠️  No rule placements found.")
        raise typer.Exit(code=1)

    typer.echo("\n📋 Optimization results:\n")

    # Console output
    for code_mod in code_mods:
        file = code_mod["target"]["file_path"]
        typer.echo("")
        typer.echo(f"File: {file} ".ljust(100, "="))
        typer.echo("Source:")
        typer.echo(f"```python\n{code_mod['target']['source_code']}\n```")
        typer.echo("Optimizations:")
        rich.print_json(data=code_mod["mods"])

    # Generate markdown report
    report_path = cwd / f"optimization_{datetime.now():%Y%m%d_%H%M}.md"
    generate_optimization_report(code_mods, write_to=report_path)
    typer.echo(f"\n📝 Optimization report saved to: {report_path}\n")


cli = typer.Typer(rich_markup_mode="markdown")


@cli.command()
def main(
    analyzer: str = "openai/o4-mini",
    evaluator: str = "openai/o4-mini",
    optimizer: str = "openai/gpt-4.1",
    synthesizer: str = "openai/gpt-4.1-nano",
    examples: int = 25,
    seed: int = 42,
    concurrency: int = 16,
):  # noqa: C901 – the CLI can be a bit long
    """Interactive `aiai` CLI as described in `aiai/cli/README.md`."""
    if examples > 25:
        typer.secho(
            "⚠️  Maximum number of synthetic examples is 25. Setting to 25.\n",
            fg=typer.colors.YELLOW,
        )
        examples = 25

    load_dotenv()

    typer.secho("\n🚀 Welcome to aiai! 🤖\n", fg=typer.colors.BRIGHT_CYAN, bold=True)
    log_init()

    # ------------------------------------------------------------------
    # 1️⃣  Agent selection
    # ------------------------------------------------------------------
    typer.secho("What would you like to optimize?\n (1) Outbound email agent (Demo)\n (2) My own agent")
    choice = typer.prompt("Enter your choice (1 or 2)", type=int)

    if choice not in (1, 2):
        typer.secho("❌ Invalid choice. Exiting.", fg=typer.colors.RED)
        raise typer.Exit(code=1)

    if choice == 1:
        if not os.getenv("OPENAI_API_KEY"):
            typer.echo("🔑 The demo agent requires an OpenAI API key...")
            if not typer.confirm("Have you added an `OPENAI_API_KEY` to the `.env` file?", default=False):
                raise typer.Exit(code=1)

        # Path to the pre‑configured demo entrypoint
        # NOTE: This path needs adjustment when run from cli/app.py
        # Let's assume the script is run from the project root for now,
        # or we pass the root path explicitly.
        # For simplicity, using a relative path from expected root execution
        log_event("demo")
        entrypoint = (Path(__file__).parent / "examples/crewai_agent.py").resolve()

    elif choice == 2:
        typer.secho(
            dedent(
                """\

                To optimize your own agent, we need an entrypoint.py file.
                This file must have a `def main(example=None)` function that
                runs your agent with the provided example.

                Here's an example of what a minimal entrypoint.py file looks like:

                ```python
                def main(example=None):
                    crew = get_crewai_agent()
                    example = example or ... # add your own default example here
                    result = crew.kickoff({"input": example})
                    return result.raw
                ```
                """
            )
        )

        log_event("custom")
        entrypoint = Path(typer.prompt("\nPath to entrypoint")).expanduser().resolve()

    # ------------------------------------------------------------------
    # 2️⃣  Entrypoint validation
    # ------------------------------------------------------------------
    try:
        _validate_entrypoint(entrypoint)
    except Exception as exc:  # noqa: BLE001 – show raw error to user
        typer.secho(f"❌ Failed to run the entrypoint. Error details:\n{exc}", fg=typer.colors.RED)
        raise typer.Exit(code=1)

    # ------------------------------------------------------------------
    # 3️⃣  Analysis setup
    # ------------------------------------------------------------------
    setup_django()
    reset_db()

    # ------------------------------------------------------------------
    # 4️⃣  Code analysis ➜ Dependency graph
    # ------------------------------------------------------------------
    with loading("Analyzing code…"):
        opt_ctx = analyze_code(entrypoint, analyzer)
    typer.echo(opt_ctx.analysis.what)
    log_event(
        "agent_context",
        analysis=opt_ctx.analysis.model_dump_json(),
        prompts=opt_ctx.optimizer_prompts.model_dump_json(),
    )

    # ------------------------------------------------------------------
    # 5️⃣  Evaluation criteria generation
    # ------------------------------------------------------------------
    try:
        # No need for inner import now
        with loading("Generating evals…"):
            generator = EvalGenerator(opt_ctx, evaluator)
            rules_eval, _ = generator.perform()  # Ignore head_to_head
        typer.echo(rules_eval.prompt.strip())
    except Exception as exc:  # noqa: BLE001 – non‑critical failure
        typer.secho(
            f"⚠️  Skipped evaluation criteria generation due to error: {exc}\n",
            fg=typer.colors.YELLOW,
        )
        rules_eval = None
    # ------------------------------------------------------------------
    # 6️⃣  Optimization run
    # ------------------------------------------------------------------
    if rules_eval:
        _optimization_run(
            entrypoint,
            rules_eval,
            context=opt_ctx,
            evaluator=evaluator,
            optimizer=optimizer,
            synthesizer=synthesizer,
            examples=examples,
            seed=seed,
            concurrency=concurrency,
        )
    else:
        typer.secho(
            "⚠️  Optimisation run skipped because evaluation criteria were not generated.\n",
            fg=typer.colors.YELLOW,
        )

    typer.secho("👋 Exiting.\n", fg=typer.colors.BRIGHT_CYAN, bold=True)


# --- Typer hook --------------------------------------------------------------
if __name__ == "__main__":
    # This allows running `python -m aiai` directly
    # Set up Django FIRST when running as main script
    # setup_django() # Now called in aiai/__init__.py
    cli()
